@@ -67,18 +67,11 @@ static int block_crypto_read_func(QCryptoBlock *block,
     BlockCrypto *crypto = bs->opaque;
     ssize_t ret;
 
-    if (qemu_in_coroutine()) {
-        GRAPH_RDLOCK_GUARD();
+    GLOBAL_STATE_CODE();
+    GRAPH_RDLOCK_GUARD_MAINLOOP();
 
-        ret = bdrv_co_pread(crypto->header ? crypto->header : bs->file,
-                            offset, buflen, buf, 0);
-    } else {
-        GLOBAL_STATE_CODE();
-        GRAPH_RDLOCK_GUARD_MAINLOOP();
-
-        ret = bdrv_pread(crypto->header ? crypto->header : bs->file,
-                         offset, buflen, buf, 0);
-    }
+    ret = bdrv_pread(crypto->header ? crypto->header : bs->file,
+                     offset, buflen, buf, 0);
     if (ret < 0) {
         error_setg_errno(errp, -ret, "Could not read encryption header");
         return ret;
@@ -97,18 +90,11 @@ static int block_crypto_write_func(QCryptoBlock *block,
     BlockCrypto *crypto = bs->opaque;
     ssize_t ret;
 
-    if (qemu_in_coroutine()) {
-        GRAPH_RDLOCK_GUARD();
+    GLOBAL_STATE_CODE();
+    GRAPH_RDLOCK_GUARD_MAINLOOP();
 
-        ret = bdrv_co_pwrite(crypto->header ? crypto->header : bs->file,
-                             offset, buflen, buf, 0);
-    } else {
-        GLOBAL_STATE_CODE();
-        GRAPH_RDLOCK_GUARD_MAINLOOP();
-
-        ret = bdrv_pwrite(crypto->header ? crypto->header : bs->file,
-                          offset, buflen, buf, 0);
-    }
+    ret = bdrv_pwrite(crypto->header ? crypto->header : bs->file,
+                      offset, buflen, buf, 0);
     if (ret < 0) {
         error_setg_errno(errp, -ret, "Could not write encryption header");
         return ret;
@@ -806,7 +792,7 @@ block_crypto_co_create_opts_luks(BlockDriver *drv, const char *filename,
     char *buf = NULL;
     int64_t size;
     bool detached_hdr =
-        qemu_opt_get_bool_del(opts, "detached-header", false);
+        qemu_opt_get_bool(opts, "detached-header", false);
     unsigned int cflags = 0;
     int ret;
     Error *local_err = NULL;
